@@ -120,17 +120,17 @@ class GraphBase:
             self.currentUsedMarker[marker] = marker
         if "marker" not in kwargs:
             kwargs["marker"] = marker
-        if "markerfacecolor" not in kwargs:
-            kwargs["markerfacecolor"] = markerfacecolor
-        if "markersize" not in kwargs:
-            kwargs["markersize"] = markersize
+        # if "markerfacecolor" not in kwargs:
+        #     kwargs["markerfacecolor"] = markerfacecolor
+        # if "markersize" not in kwargs:
+        #     kwargs["markersize"] = markersize
         # 默认自动选择 color
         if color == "&&":
             color = self.autoSelectColor()
             self.currentUsedColor[color] = color
         if "color" not in kwargs:
             kwargs["color"] = color
-        plt.plot(x, y, *args, **kwargs)
+        plt.scatter(x, y, *args, **kwargs)
         return self
 
     def innerPlotSum(self, nodeList: [NodeItem], getNode, getX, getY, *args,
@@ -238,6 +238,7 @@ class GraphBase:
 
     def close(self):
         plt.close()
+        return self
 
 
 class ExportItem:
