@@ -119,7 +119,8 @@ class NodeAppDelay:
             return
         self.typeMap[item.Type.name].append(item)
 
-    def getX(self, delayType: DelayType = DelayType.LastDelay, samplingInterval: float = 1.0):
+    def getX(self, delayType: DelayType = DelayType.LastDelay,
+             samplingInterval: float = 1.0, raw: bool = False):
         """
         获取采样时间列表
         :param delayType:
@@ -127,17 +128,46 @@ class NodeAppDelay:
         :return:
         """
         lastCount, res = 0, []
-        for item in self.typeMap[delayType.name]:
-            if int(item.Time / samplingInterval) != lastCount:
-                lastCount = int(item.Time / samplingInterval)
-            else:
+        for i in range(len(self.typeMap[DelayType.FullDelay.name])):
+            fullItem, lastItem = self.typeMap[DelayType.FullDelay.name][i], \
+                                 self.typeMap[DelayType.LastDelay.name][i]
+            full, last = fullItem.getValueByDelayTarget(DelayTarget.DelayMS), \
+                         lastItem.getValueByDelayTarget(DelayTarget.DelayMS)
+            if full - last > 10e-6:
                 continue
-            res.append(lastCount * samplingInterval)
+            item = self.typeMap[delayType.name][i]
+            if samplingInterval > 0 and not raw:
+                if int(item.Time / samplingInterval) != lastCount:
+                    lastCount = int(item.Time / samplingInterval)
+                else:
+                    continue
+                res.append(lastCount * samplingInterval)
+            else:
+                if full - last < 10e-6:
+                    res.append(fullItem.Time)
+        # for item in self.typeMap[delayType.name]:
+        #     full, last = item.getValueByDelayTarget(DelayType.FullDelay),\
+        #                  item.getValueByDelayTarget(DelayType.LastDelay)
+        #     if full - last < 10e-6:
+        #         # 只提取非重传的
+        #         res.append(item.Time)
+
+        # if int(item.Time / samplingInterval) != lastCount:
+        #     lastCount = int(item.Time / samplingInterval)
+        # else:
+        #     continue
+        # res.append(lastCount * samplingInterval)
+
+        # if int(item.Time / samplingInterval) != lastCount:
+        #     lastCount = int(item.Time / samplingInterval)
+        # else:
+        #     continue
+        # res.append(lastCount * samplingInterval)
         return res
 
     def getY(self, delayType: DelayType = DelayType.LastDelay,
              delayTarget: DelayTarget = DelayTarget.DelayMS,
-             samplingInterval: float = 1.0):
+             samplingInterval: float = 1.0, raw: bool = False):
         """
         获取延迟采样列表
         :param delayType:           延迟类型
@@ -146,15 +176,43 @@ class NodeAppDelay:
         :return:
         """
         lastCount, res, tmp = 0, [], []
-        for item in self.typeMap[delayType.name]:
-            if int(item.Time / samplingInterval) != lastCount:
-                tmp.append(item.getValueByDelayTarget(delayTarget))
-                lastCount = int(item.Time / samplingInterval)
-            else:
-                tmp.append(item.getValueByDelayTarget(delayTarget))
+        for i in range(len(self.typeMap[DelayType.FullDelay.name])):
+            fullItem, lastItem = self.typeMap[DelayType.FullDelay.name][i], \
+                                 self.typeMap[DelayType.LastDelay.name][i]
+            full, last = fullItem.getValueByDelayTarget(DelayTarget.DelayMS), \
+                         lastItem.getValueByDelayTarget(DelayTarget.DelayMS)
+
+            if full - last > 10e-6:
                 continue
-            res.append(np.average(tmp))
-            tmp = []
+            item = self.typeMap[delayType.name][i]
+            if samplingInterval > 0 and not raw:
+                # if int(item.Time / samplingInterval) != lastCount:
+                #     lastCount = int(item.Time / samplingInterval)
+                # else:
+                #     tmp.append(item.getValueByDelayTarget(delayTarget))
+                #     continue
+                # res.append(np.average(tmp))
+                # tmp = []
+
+                if int(item.Time / samplingInterval) != lastCount:
+                    res.append(item.getValueByDelayTarget(delayTarget))
+                    lastCount = int(item.Time / samplingInterval)
+                else:
+                    continue
+            else:
+                res.append(last)
+        # for item in self.typeMap[delayType.name]:
+        #     full, last = item.getValueByDelayTarget(DelayType.FullDelay), \
+        #                  item.getValueByDelayTarget(DelayType.LastDelay)
+        #     if full - last < 10e-6:
+        #         res.append(last)
+
+        # if int(item.Time / samplingInterval) != lastCount:
+        #     res.append(item.getValueByDelayTarget(delayTarget))
+        #     lastCount = int(item.Time / samplingInterval)
+        # else:
+        #     continue
+
         return res
 
 
@@ -180,7 +238,8 @@ class NodeDelay:
             self.appMap[item.AppId] = NodeAppDelay(item.Node, item.AppId)
         self.appMap[item.AppId].appendItem(item)
 
-    def getX(self, AppId: int, delayType: DelayType = DelayType.LastDelay, samplingInterval: float = 1.0):
+    def getX(self, AppId: int, delayType: DelayType = DelayType.LastDelay,
+             samplingInterval: float = 1.0, raw: bool = False):
         """
         获取某个App的采样时间列表
         :param AppId:
@@ -190,11 +249,11 @@ class NodeDelay:
         """
         if AppId not in self.appMap:
             return []
-        return self.appMap[AppId].getX(delayType, samplingInterval)
+        return self.appMap[AppId].getX(delayType, samplingInterval, raw)
 
     def getY(self, AppId: int, delayType: DelayType = DelayType.LastDelay,
              delayTarget: DelayTarget = DelayTarget.DelayMS,
-             samplingInterval: float = 1.0):
+             samplingInterval: float = 1.0, raw: bool = False):
         """
         获取某个App的延迟采样列表
         :param AppId:
@@ -205,7 +264,7 @@ class NodeDelay:
         """
         if AppId not in self.appMap:
             return []
-        return self.appMap[AppId].getY(delayType, delayTarget, samplingInterval)
+        return self.appMap[AppId].getY(delayType, delayTarget, samplingInterval, raw)
 
 
 class Delay:
@@ -253,6 +312,7 @@ class DelayExtractBase:
         self.delayType = DelayType.LastDelay
         self.delayTarget = DelayTarget.DelayMS
         self.delay = delay
+        self.raw = False
 
     def setSamplingInterval(self, samplingInterval: float):
         """
@@ -263,6 +323,10 @@ class DelayExtractBase:
         self.samplingInterval = samplingInterval
         return self
 
+    def setRaw(self, raw: bool):
+        self.raw = raw
+        return self
+
     def setDelayType(self, delayType: DelayType):
         """
         设置延迟类型
@@ -285,10 +349,11 @@ class DelayExtractBase:
         return self.delay.getByNode(nodeName)
 
     def getX(self, node: NodeDelay, nodeId):
-        return node.getX(nodeId, samplingInterval=self.samplingInterval)
+        return node.getX(nodeId, samplingInterval=self.samplingInterval, raw=self.raw)
 
     def getY(self, node: NodeDelay, nodeId):
-        return node.getY(nodeId, self.delayType, self.delayTarget, self.samplingInterval)
+        return node.getY(nodeId, self.delayType, self.delayTarget,
+                         self.samplingInterval, self.raw)
 
     def getUnit(self):
         return DelayTarget.getUnit(self.delayTarget.name)
@@ -366,6 +431,81 @@ class DelayGraph(GraphBase, DelayExtractBase, ExportBase):
                           **kwargs)
         return self
 
+    def exportExcel(self, savePath: str,
+                    xName: str = "Times",
+                    yName: str = "Delay",
+                    xUnit: str = "Second"):
+        """
+        导出数据到Excel
+        :param savePath:
+        :param xName:
+        :param yName:
+        :param xUnit:
+        :return:
+        """
+        dfResult = None
+        for item in self.items:
+            contents = [[xName, yName], [xUnit, item.units], ["", item.label]]
+            dataMatrix = np.vstack([np.array(item.x).T, np.array(item.y).T])
+            for i in range(len(item.x)):
+                contents.append(dataMatrix[:, i])
+            df = pd.DataFrame(contents)
+            if dfResult is None:
+                dfResult = df
+            else:
+                dfResult = pd.concat([dfResult, df], axis=1)
+        dfResult.to_excel(savePath, index=False, header=None)
+        return self
+
+    def exportGroupExcel(self, savePath: str):
+        dfResult = None
+        for item in self.items:
+            nextCount = int(item.x[0] / self.samplingInterval) * self.samplingInterval + self.samplingInterval
+            # 填充没有的时间
+            for i in range(int(item.x[0] / self.samplingInterval)):
+                res = [[(i + 1) * self.samplingInterval], [item.label], []]
+                df = pd.DataFrame(res)
+                if dfResult is None:
+                    dfResult = df
+                else:
+                    dfResult = pd.concat([dfResult, df], axis=1)
+            tmp = [[nextCount], [item.label], []]
+            for i in range(len(item.x)):
+                if item.x[i] < nextCount:
+                    tmp.append([item.y[i]])
+                else:
+                    df = pd.DataFrame(tmp)
+                    if dfResult is None:
+                        dfResult = df
+                    else:
+                        dfResult = pd.concat([dfResult, df], axis=1)
+                    nextCount += self.samplingInterval
+                    tmp = [[nextCount], [item.label], [], [item.y[i]]]
+            if len(tmp) > 10:
+                dfResult = pd.concat([dfResult, pd.DataFrame(tmp)], axis=1)
+        dfResult.to_excel(savePath, index=False, header=None)
+        return self
+
+    def exportGroupCategoriesExcel(self, savePath: str):
+        dfResult = None
+        tmp = [['Time', 'Consumer', 'Data'], ['', '', ''], ['', '', '']]
+        for item in self.items:
+            nextCount = int(item.x[0] / self.samplingInterval) * self.samplingInterval + self.samplingInterval
+            # 填充没有的时间
+            for i in range(int(item.x[0] / self.samplingInterval)):
+                res = [[(i + 1) * self.samplingInterval], [item.label], []]
+                tmp.append([self.samplingInterval * (i + 1), item.label, 0])
+
+            for i in range(len(item.x)):
+                tmp.append([nextCount, item.label, item.y[i]])
+                if item.x[i] < nextCount:
+                    continue
+                else:
+                    nextCount += self.samplingInterval
+
+        pd.DataFrame(tmp).to_excel(savePath, index=False, header=None)
+        return self
+
     @staticmethod
     def parse(inputFile: str):
         return DelayGraph(Delay.parse(inputFile))
