@@ -362,25 +362,7 @@ class ThroughputExtractBase:
         return ThroughputTarget.getUnit(self.throughputTarget.name)
 
 
-class ThroughputExport(ThroughputExtractBase, ExportBase):
-    """
-    一个用于导出数据到Excel的辅助类
-    """
-
-    def __init__(self, throughput: Throughput):
-        ThroughputExtractBase.__init__(self, throughput)
-        ExportBase.__init__(self, self.getNode, self.getX, self.getY, self.getUnit)
-
-    def innerParse(self, inputFile: str):
-        self.throughput = Throughput.parse(inputFile)
-        return self
-
-    @staticmethod
-    def parse(inputFile: str):
-        return ThroughputExport(Throughput.parse(inputFile))
-
-
-class ThroughputGraph(GraphBase, ThroughputExtractBase):
+class ThroughputGraph(GraphBase, ThroughputExtractBase, ExportBase):
     """
     一个用于实现绘制吞吐量图的类
     """
@@ -388,13 +370,14 @@ class ThroughputGraph(GraphBase, ThroughputExtractBase):
     def __init__(self, throughput: Throughput):
         GraphBase.__init__(self)
         ThroughputExtractBase.__init__(self, throughput)
+        ExportBase.__init__(self, self.getNode, self.getX, self.getY, self.getUnit)
 
-    def plot(self, node: str, faceId: int, *args,
+    def plot(self, nodeName: str, faceId: int, *args,
              color: str = "&&",
              linewidth: float = 2, linestyle: str = "dotted", marker: str = "&&",
              markerfacecolor: str = "none", markersize: float = 6,
              **kwargs):
-        node = self.throughput.getByNode(node)
+        node = self.throughput.getByNode(nodeName)
         if not node:
             print("not exist node: ", node)
             return self
@@ -403,6 +386,7 @@ class ThroughputGraph(GraphBase, ThroughputExtractBase):
         x, y = node.getX(faceId, samplingInterval=self.samplingInterval), \
                node.getY(faceId, self.throughputType, self.throughputTarget, self.samplingInterval,
                          self.lpPacketHeaderSize)
+        self.select(nodeName, faceId, kwargs["label"])
         super().innerPlot(x, y, *args,
                           color=color,
                           linewidth=linewidth,
@@ -418,7 +402,10 @@ class ThroughputGraph(GraphBase, ThroughputExtractBase):
                 linewidth: float = 2, linestyle: str = "dotted", marker: str = "&&",
                 markerfacecolor: str = "none", markersize: float = 6,
                 **kwargs):
-
+        label = None
+        if "label" in kwargs:
+            label = kwargs["label"]
+        self.selectSum(nodeList, label=label)
         super().innerPlotSum(nodeList, self.getNode, self.getX, self.getY, *args,
                              color=color,
                              linewidth=linewidth,
@@ -434,7 +421,10 @@ class ThroughputGraph(GraphBase, ThroughputExtractBase):
                 linewidth: float = 2, linestyle: str = "dotted", marker: str = "&&",
                 markerfacecolor: str = "none", markersize: float = 6,
                 **kwargs):
-
+        label = None
+        if "label" in kwargs:
+            label = kwargs["label"]
+        self.selectAvg(nodeList, label=label)
         self.innerPlotAvg(nodeList, self.getNode, self.getX, self.getY, *args,
                           color=color,
                           linewidth=linewidth,
